unit DockMemoFormExpert;

interface

uses 
  Sysutils, Classes, ToolsAPI, Menus, DockMemoForm;

type
  TDockMemoFormExpert = class(TNotifierObject, IOTANotifier, IOTAWizard)
  public
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
    procedure Destroyed;
  end;

  TIDENotifier = class(TNotifierObject, IOTAIDENotifier)
  private
    procedure FileNotification(NotifyCode: TOTAFileNotification;
      const FileName: string; var Cancel: Boolean);
    procedure BeforeCompile(const Project: IOTAProject; var Cancel: Boolean); overload;
    procedure AfterCompile(Succeeded: Boolean); overload;
  end;

  procedure Register;

implementation

var
  MemoMenu: TMenuItem;
  IDENotifierIndex: Integer;
  ProjectFileName: string;
  MemoFileName: string;

procedure Register;
begin
  RegisterPackageWizard(TDockMemoFormExpert.Create);
  TDockableMemoForm.CreateDockMemoForm;
end; { Register }

procedure ChangeFile;
begin
  if GetActiveProject = nil then
    begin
      ProjectFileName := '';
      MemoFileName    := '';
    end
  else
    begin
      ProjectFileName := GetActiveProject.FileName;
      MemoFileName    := ChangeFileExt(ProjectFileName, '.memo');
    end;
end; { ChangeFile }

procedure LoadFile;
begin
  with TDockableMemoForm(FormInstance) do
    if FileExists(ProjectFileName) and FileExists(MemoFileName) then
      mmMemo.Lines.LoadFromFile(MemoFileName{$IFDEF UNICODE}, TEncoding.UTF8{$ENDIF})
    else
      mmMemo.Lines.Clear;
end; { LoadFile }

procedure MenuClick(Self, Sender: TObject);
var
  HasForm: Boolean;
begin
  HasForm := Assigned(FormInstance);
  TDockableMemoForm.ShowDockMemoForm;
  if not HasForm then
    begin
      ChangeFile;
      if Assigned(FormInstance) then
        LoadFile;
    end;
end; { MenuClick }

procedure RegisterMenu;
var
  NTA: INTAServices;
  i, j: Integer;
  Method: TMethod;
  NewMenu: Boolean;
begin
  NTA := BorlandIDEServices as INTAServices;
  for i := 0 to NTA.MainMenu.Items.Count-1 do
    begin
      if NTA.MainMenu.Items[i].Name <> 'ViewsMenu' then
        Continue;
      for j := 0 to NTA.MainMenu.Items[i].Count-1 do
        begin
          if NTA.MainMenu.Items[i][j].Name = 'ViewToolWindowsItem' then
            NewMenu := True
          else if NTA.MainMenu.Items[i][j].Name = 'ViewToggleSeparator' then
            NewMenu := False
          else
            Continue;
          MemoMenu := TMenuItem.Create(nil);
          MemoMenu.AutoHotkeys := maManual;
          {$IF CompilerVersion >= 21.0}
          if not SameText(PreferredUILanguages, 'JA') then
            MemoMenu.Caption := Title_EN
          else
          {$IFEND}
            MemoMenu.Caption := Title_JA;
          Method.Code := @MenuClick;
          Method.Data := MemoMenu;
          MemoMenu.OnClick := TNotifyEvent(Method);
          if NewMenu then
            NTA.MainMenu.Items[i][j].Add(MemoMenu)
          else
            NTA.MainMenu.Items[i].Insert(j, MemoMenu);
          Break;
        end;
      Break;
    end;
end; { RegisterMenu }

procedure RegisterNotifier;
begin
  IDENotifierIndex := (BorlandIDEServices as IOTAServices).AddNotifier(TIDENotifier.Create);
end; { RegisterNotifier }

procedure SaveFile;
begin
  with TDockableMemoForm(FormInstance) do
    if FileExists(ProjectFileName) then
      mmMemo.Lines.SaveToFile(MemoFileName{$IFDEF UNICODE}, TEncoding.UTF8{$ENDIF});
end; { SaveFile }

procedure SetCaption;
var
  CaptionText, ProjectText: string;
begin
  {$IF CompilerVersion >= 21.0}
  if not SameText(PreferredUILanguages, 'JA') then
    CaptionText := Title_EN
  else
  {$IFEND}
    CaptionText := Title_JA;
  if GetActiveProject = nil then
    ProjectText := ''
  else
    ProjectText := ExtractFileName(GetActiveProject.FileName) + ' - ';
  TDockableMemoForm(FormInstance).Caption := Format('%s%s', [ProjectText, CaptionText]);
end; { SetCaption }

procedure UnregisterMenu;
begin
  MemoMenu.Free;
end; { UnregisterMenu }

procedure UnregisterNotifier;
begin
  (BorlandIDEServices as IOTAServices).RemoveNotifier(IDENotifierIndex);
end; { UnregisterNotifier }

procedure WriteLog(TabName, s: string);
var
  IMessageServices: IOTAMessageServices;
  MessageGroup: IOTAMessageGroup;
begin
  IMessageServices := BorlandIDEServices as IOTAMessageServices;
  MessageGroup := IMessageServices.AddMessageGroup(TabName);
  IMessageServices.AddTitleMessage(s , MessageGroup);
end; { WriteLog }

{ TDockMemoFormExpert }

procedure TDockMemoFormExpert.Destroyed;
begin
  TDockableMemoForm.RemoveDockMemoForm;
end;

procedure TDockMemoFormExpert.Execute;
begin
  TDockableMemoForm.ShowDockMemoForm;
end;

function TDockMemoFormExpert.GetIDString: string;
begin
  Result := 'DockingMemoFormExpert';
end;

function TDockMemoFormExpert.GetName: string;
begin
  Result := 'Docking Memo Form Expert';
end;

function TDockMemoFormExpert.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

{ TIDENotifier }

procedure TIDENotifier.AfterCompile(Succeeded: Boolean);
begin
  { Do nothing }
end;

procedure TIDENotifier.BeforeCompile(const Project: IOTAProject;
  var Cancel: Boolean);
begin
  { Do nothing }
end;

procedure TIDENotifier.FileNotification(NotifyCode: TOTAFileNotification;
  const FileName: string; var Cancel: Boolean);
begin
  if not Assigned(FormInstance) then
    Exit;
  case NotifyCode of
    ofnFileClosing:
      begin
        if GetActiveProject = nil then
          begin
            SaveFile;
            ChangeFile;
            SetCaption;
            TDockableMemoForm(FormInstance).mmMemo.Lines.Clear;
          end;
      end;
    ofnActiveProjectChanged:
      begin
        SaveFile;
        ChangeFile;
        SetCaption;
        LoadFile;
      end;
  end;
end;

initialization
  RegisterMenu;
  RegisterNotifier;

finalization
  UnregisterNotifier;
  UnregisterMenu;
  TDockableMemoForm.RemoveDockMemoForm;

end.
